#!/bin/sh

#
# Recoded for sh July 2005 to dodge a possible bug in tcsh 6.14. 
#
# Takes a list of files as input, and echos those that appear to be main() files
# of C programs. 
#
# This script is used in several makefiles and the script dirs_for_make. 
#

read file

while [ "${file}" != "" ]
do
    if [ -e "${file}" ]; then 
        #
        # We strip comments, so a commented out main() will not cause trouble.
        # However, this will still fail if main() is #ifdef'd out. Getting this
        # correct involves running the pre-processor with the same args that the
        # program will be compiled with, and is a bit awkward to rely on.
        #
        # Hence, I have implemented two different approaches.
        #
        # If the script has arguments, then they are assumed to be suitable for
        # asking the pre-processer to remove comments and all code not relavent
        # to determining of this file has a bonafide main or not. 
        #
        # If the script has no arguments, then all pre-processor commands are
        # stripped before using the pre-processer, which gets rid of comments,
        # but can make it os that an #ifdef'd out main is assumed to be a live
        # one. 
        #
        if [ $# == 0 ]; then 
            test_main=`grep -v '[ 	]*#'  ${file} | cpp | grep '^ *\(int\)* *\(void\)* *main *(.*)'`
        else
            test_main=`cpp $*  ${file} | grep '^ *\(int\)* *\(void\)* *main *(.*)'`
        fi

        if [ "${test_main}" == "" ]; then
            echo $file
        fi
    fi 

    read file
done 

exit 0


